#!/usr/bin/env python
# Copyright 2009 David D Lowe
# released under the GPLv3

from pigment import PigmentType, Pigment
from pigment_more import DirBasedPigment, DirBasedPigmentType
import pigment
import os, sys
import subprocess
from glob import glob
if __name__ == "__main__" or 'pychecker' in sys.argv[0]:
    sys.path.append("..")
from const import DEBUG
import ep_exceptions
import handy
import const
import hashlib
from gettext import gettext as _

class XsplashHackType(DirBasedPigmentType):
    humanName = {"en":"Xsplash"}
    humanPluralName = {"en":"Xsplashes"}
    codeName = "xsplashHack"
    abbreviation = "xsplh"
    directoryName = "xsplashesHack"
    canGeneratePreview = True
    
    attachmentTypeHelp = _("The theme file should be a directory with the files that need to be replaced in /usr/share/images/xplash")
    
    def get_Pigment(self):
        return XsplashHack
    
    publicBaseDir = "/usr/share/images/xsplash"
    personalBaseDir = ""
    
    themeFiles = ("bg_1024x768.jpg","bg_1440x900.jpg","bg_1920x1200.jpg","bg_800x600.jpg","logo_medium.png","logo_xtra_large.png","throbber_large.png","throbber_small.png",
        "bg_1280x1024.jpg","bg_1680x1050.jpg","bg_2560x1600.jpg","logo_large.png","logo_small.png","throbber_medium.png","throbber_xtra_large.png")
        

class XsplashHack(DirBasedPigment):
    
    
    def __init__(self):
        DirBasedPigment.__init__(self)
        self.type = XsplashHackType()
        
        self.preferredGtkThemes = []
        self.preferredIconThemes = []
        self.preferredCursorThemes = []
        self.xsplashFrames = None
    
    def install(self, installationDirectory=None, progressCallback=None, shell=None):
        exitPlease = False
        if progressCallback is None: progressCallback = lambda progress, query: False
        def change_progress_callback(progress, query):
            if progressCallback(progress / 2, query):
                exitPlease = True
                return True
            return False
        # copy pigment information over and copy file
        DirBasedPigment.install(self, installationDirectory, change_progress_callback, shell)
        if exitPlease: return
        
        if progressCallback(0.6, None): return
        needToCreateBackup = False
        for filename in self.type.themeFiles:
            if not os.path.islink(os.path.join(self.type.publicBaseDir,filename)):
                needToCreateBackup = True
                break
        if needToCreateBackup:
            # create backup at /usr/share/images/xsplash/original
            shell.do(["mkdir","-p",os.path.join(self.type.publicBaseDir,"original")])
            for filename in self.type.themeFiles:
                shell.do(["mv",os.path.join(self.type.publicBaseDir, filename),
                    os.path.join(self.type.publicBaseDir,"original/")])
                shell.do(["ln","-s","original/" + filename,"-f",filename],cwd=self.type.publicBaseDir)
        # TODO: change throbber settings
        progressCallback(1.0, None)
            
    
    def remove(self, progressCallback=None, shell=None):
        exitPlease = False
        def change_progress_callback(progress):
            if progressCallback(progress / 2):
                exitPlease = True
                return True
            return False
        # remove pigment information and directory
        DirBasedPigment.remove(self, change_progress_callback, shell)
        if exitPlease: return
        if progressCallback(0.6): return
        # see if xsplash theme needs reverting back to original
        shouldRevert = False
        for filename in self.type.themeFiles:
            if not os.path.exists(os.path.realpath(os.path.join(self.type.publicBaseDir,filename))):
                shouldRevert = True
                break
        if shouldRevert:
            # link back to original files
            for filename in self.type.themeFiles:
                shell.do(["ln","-s","original/" + filename,"-f",filename],cwd=self.type.publicBaseDir)
            # change /etc/gdm/Init/Default back to orig
            ff = open("/etc/gdm/Init/Default","r")
            output = ""
            for line in ff.readlines():
                if line.strip()[:39] =="/usr/bin/xsplash --gdm-session --daemon":
                    countSpaces = 0
                    for ii in xrange(len(line)):
                        if line[ii] == " ":
                            countSpaces += 1
                        else:
                            break
                    output += (" " * countSpaces) + "/usr/bin/xsplash --gdm-session --daemon\n"
                else:
                    output += line
            ff.close()
            subprocess.call(["mkdir","-p","/tmp/epidermis"])
            subprocess.call(["touch","/tmp/epidermis/gdm-file"])
            ff = open("/tmp/epidermis/gdm-file","w")
            ff.write(output)
            ff.close()
            shell.do(["mv","-f","/tmp/epidermis/gdm-file","/etc/gdm/Init/Default"])
            
        progressCallback(1.0)
    
    def activate(self, progressCallback=None, shell=None):
        if progressCallback is None:
            progressCallback = lambda(a1): False
        if progressCallback(0.0): return
        # link to all the new files
        for filename in self.type.themeFiles:
            shell.do(["ln","-s",self.attDirname + "/" + filename,"-f",filename],cwd=self.type.publicBaseDir)
        if progressCallback(0.5): return
        for theme in self.preferredGtkThemes:
            if os.path.isdir(os.path.join("/usr/share/themes",theme,"gtk-2.0")):
                if DEBUG:
                    print "setting gdm's gtk theme to %s" % theme
                shell.do(["sudo","-u","gdm","gconftool-2","-t","string","-s","/desktop/gnome/interface/gtk_theme",theme])
                break
        for theme in self.preferredIconThemes:
            if os.path.isdir(os.path.join("/usr/share/icons",theme)):
                if DEBUG:
                    print "setting gdm's icon theme to %s" % theme
                shell.do(["sudo","-u","gdm","gconftool-2","-t","string","-s","/desktop/gnome/interface/icon_theme",theme])
                break
        for theme in self.preferredCursorThemes:
            if os.path.isdir(os.path.join("/usr/share/icons",theme,"cursors")):
                if DEBUG:
                    print "setting gdm's cursor theme to %s" % theme
                shell.do(["sudo","-u","gdm","gconftool-2","-t","string","-s","/desktop/gnome/peripherals/mouse/cursor_theme",theme])
                break
        
        # xsplash frames, modify /etc/gdm/Init/Default
        if DEBUG:
            print "writing xsplash frames", self.xsplashFrames
        output = ""
        ff = open("/etc/gdm/Init/Default","r")
        modifiedByEpidermis = False
        for line in ff.readlines():
            if line.strip() == "##EPIDERMIS HAS MODIFIED THIS FILE, REINSTALL gdm TO UNDO CHANGES##":
                modifiedByEpidermis = True
                output += line
            elif line.strip()[:39] =="/usr/bin/xsplash --gdm-session --daemon":
                frames = self.xsplashFrames
                if not modifiedByEpidermis:
                    output += "##EPIDERMIS HAS MODIFIED THIS FILE, REINSTALL gdm TO UNDO CHANGES##\n"
                countSpaces = 0
                for ii in xrange(len(line)):
                    if line[ii] == " ":
                        countSpaces += 1
                    else:
                        break
                output += (" " * countSpaces) + "/usr/bin/xsplash --gdm-session --daemon"
                if not frames is None:
                    output += " -f %i" % frames
                output += "\n"
            else:
                output += line
        if len(output) > 0: output = output[:-1]
        ff.close()
        subprocess.call(["mkdir","-m","777","-p","/tmp/epidermis"])
        subprocess.call(["touch","/tmp/epidermis/gdm-file"])
        ff = open("/tmp/epidermis/gdm-file","w")
        ff.write(output)
        ff.close()
        shell.do(["mv","-f","/tmp/epidermis/gdm-file","/etc/gdm/Init/Default"])
        progressCallback(1.0)
    
    def read(self, filePath):
        DirBasedPigment.read(self, filePath)
        
        from xml.dom import minidom
        doc = minidom.parse(filePath)
        root = doc.getElementsByTagName("pigment")[0]
        
        for par, child,prop in (("preferredGtkThemes","gtkTheme",self.preferredGtkThemes),
        ("preferredIconThemes","iconTheme",self.preferredIconThemes), ("preferredCursorThemes",
        "cursorTheme",self.preferredCursorThemes)):
            del prop[:]
            parXml = root.getElementsByTagName(par)[0]
            for childXml in parXml.getElementsByTagName(child):
                prop.append(pigment.get_first_text(childXml))
        
        try:
            self.xsplashFrames = handy.get_first_text(root.getElementsByTagName("xsplashFrames")[0])
            self.xsplashFrames = int(self.xsplashFrames)
        except ep_exceptions.NoFirstTextError:
            self.xsplashFrames = None
        
    
    def information_xml(self):
        """Returns the XML document describing the pigment"""
        doc = DirBasedPigment.information_xml(self)
        root = doc.getElementsByTagName("pigment")[0]
        
        for par, child,prop in (("preferredGtkThemes","gtkTheme",self.preferredGtkThemes),
        ("preferredIconThemes","iconTheme",self.preferredIconThemes), ("preferredCursorThemes",
        "cursorTheme",self.preferredCursorThemes)):
            xmlPar = doc.createElement(par)
            for theme in prop:
                xmlTheme = doc.createElement(child)
                tn = doc.createTextNode(theme)
                xmlTheme.appendChild(tn)
                xmlPar.appendChild(xmlTheme)
            root.appendChild(xmlPar)
        
        xmlBit = doc.createElement("xsplashFrames")
        text = str(self.xsplashFrames)
        if self.xsplashFrames is None:
            text = ""
        tn = doc.createTextNode(text)
        xmlBit.appendChild(tn)
        root.appendChild(xmlBit)
        
        return doc

    def is_activated(self):
        """Returns whether this pigment is applied. To check if the effects
        have been activated regardless of Epidermis' pigments, use is_meat_activated"""
        return self.is_meat_activated()
    
    def is_attachment_correct(self, attachmentIndex=None):
        if DirBasedPigment.is_attachment_correct(self, attachmentIndex):
            if attachmentIndex is None:
                attachmentIndex = 0
            for filename in self.type.themeFiles:
                if not os.path.exists(os.path.join(self.attPath, filename)):
                    return False
            return True
        else: return False
    
    def generate_preview(self, answerYes=None):
        handy.require_main_thread()
        if not answerYes:
            import gtk
            dialog = gtk.MessageDialog(None, gtk.DIALOG_MODAL, \
                gtk.MESSAGE_QUESTION, gtk.BUTTONS_YES_NO, "Previewing this xsplash theme will activate it. Continue?")
            rr = dialog.run()
            print "rr", rr
            import gobject
            gobject.idle_add(lambda:dialog.hide())
            if rr != gtk.RESPONSE_YES:
                return None
        import shell
        shll = shell.BashRootShell()
        shll.prepare()
        
        if not shll.ready:
            return None
        needToCreateBackup = False
        for filename in self.type.themeFiles:
            if not os.path.islink(os.path.join(self.type.publicBaseDir,filename)):
                needToCreateBackup = True
                break
        if needToCreateBackup:
            # create backup at /usr/share/images/xsplash/original
            shll.do(["mkdir","-p",os.path.join(self.type.publicBaseDir,"original")])
            for filename in self.type.themeFiles:
                shll.do(["mv",os.path.join(self.type.publicBaseDir, filename),
                    os.path.join(self.type.publicBaseDir,"original/")])
                shll.do(["ln","-s","original/" + filename,"-f",filename],cwd=self.type.publicBaseDir)
        shll.do(["cp","-r",self.attPath,self.type.publicBaseDir])
        for filename in self.type.themeFiles:
            shll.do(["ln","-s",self.attDirname + "/" + filename,"-f",filename],cwd=self.type.publicBaseDir)
                    
        if not self.xsplashFrames is None:
            shll.do("xsplash -f %d -x 4 &" % self.xsplashFrames, intrShell=True)
        else:
            shll.do("xsplash -x 4 &", intrShell=True)
            
        import gtk.gdk
        import time
        time.sleep(2)

        w = gtk.gdk.get_default_root_window()
        sz = w.get_size()
        if const.DEBUG:
            print "The size of the window is %d x %d" % sz
        pb = gtk.gdk.Pixbuf(gtk.gdk.COLORSPACE_RGB,False,8,sz[0],sz[1])
        pb = pb.get_from_drawable(w,w.get_colormap(),0,0,0,0,sz[0],sz[1])
        if not (pb is None):
            pb = pb.scale_simple(100,75, gtk.gdk.INTERP_BILINEAR)
            self.preview = pb
            return pb
        else:
            return None
    
    def is_meat_activated(self):
        """Returns whether the meat of this pigment or similar to it is activated"""
        # TODO: finish is_meat_activated
        pass
    
    def does_activation_require_root(self):
        """Returns whether the activation requires root rights.
        If it does, the pigment object will need a pexpect bash shell as root
        to be passed to some of its methods"""
        return True
    
    def get_display_data(self, index=0):
        """Returns the object of index to be added to a gtk.ListStore, this
        could be a string, a gtk.gdk.Pixbuf or a custom widget, it should
        be rendered using type.create_cell_renderer"""
        return

if __name__ == "__main__":
    print __file__,"should not be run"
